#include <stdio.h>
#include <stdlib.h>

void naive_matrix_multiply(int matrix_size,
                           double *a, double *b, double *c, int quiet) {

  double sum;
  int i,j,k;

  /* initialize matrix to arbitrary initial values */
  /* a[i][j]=i*j     */
  /* b[i][j]=i/(j+5) */
  /* c[i][j]=0.0     */
  /* Purposefully a non-symmetric matrix so we can more easily see */
  /* implementation errors.                                        */

  for(i=0;i<matrix_size;i++) {
    for(j=0;j<matrix_size;j++) {
       a[(i*matrix_size)+j]=(double)i*(double)j;
       b[(i*matrix_size)+j]=(double)i/(double)(j+5);
       c[(i*matrix_size)+j]=0.0;
    }
  }

  /* Do the matrix-matrix multiplication */

  int rand_i,rand_j,rand_k;
  for(i=0;i<matrix_size;i++) {
     for(k=0;k<matrix_size;k++) {
        for(j=0;j<matrix_size;j++) {
	   rand_i=rand()%matrix_size;
           rand_j=rand()%matrix_size;
           rand_k=rand()%matrix_size;

	   c[(i*matrix_size)+j]+=a[(rand_i*matrix_size+rand_k)]*
                                 b[(rand_k*matrix_size)+rand_j];
	}
     }
  }

  /* Sum the matrix to calculate a result we can compare */
  sum=0.0;
  for(i=0;i<matrix_size;i++) {
    for(j=0;j<matrix_size;j++) {
      sum+=c[(i*matrix_size)+j];
    }
  }

  if (!quiet) printf("Matrix multiply sum: s=%lf\n",sum);

  return;
}

int main(int argc, char **argv) {

   int matrix_size=512;
   double *a,*b,*c;

   if (argc>1) matrix_size=atoi(argv[1]);

   printf("Using a matrix size of %d x %d\n",matrix_size,matrix_size);

   a=calloc(matrix_size*matrix_size,sizeof(double));
   b=calloc(matrix_size*matrix_size,sizeof(double));
   c=calloc(matrix_size*matrix_size,sizeof(double));
   
   if ((a==NULL) || (b==NULL) || (c==NULL)) {
      fprintf(stderr,"Error allocating memory for matrices!\n");
      return -1;
   }

   naive_matrix_multiply(matrix_size,a,b,c,0);
   
   return 0;
}
